@@ -3,7 +3,7 @@
 
 #include <rz_asm.h>
 #include <rz_lib.h>
-#include <capstone.h>
+#include <capstone/capstone.h>
 
 #ifdef CAPSTONE_M68K_H
 #define CAPSTONE_HAS_M68K 1
@@ -17,7 +17,7 @@
 #endif
 
 #if CAPSTONE_HAS_M68K
-#include <m68k.h>
+#include <capstone/m68k.h>
 // http://www.mrc.uidaho.edu/mrc/people/jff/digital/M68Kir.html
 
 #define OPERAND(x)  insn->detail->m68k.operands[x]
@@ -166,7 +166,7 @@ static void op_fillval(RzAnalysisOp *op, csh handle, cs_insn *insn) {
 	}
 }
 
-static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, int len, RzAnalysisOpMask mask) {
+static int analyze_op(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, int len, RzAnalysisOpMask mask) {
 	int n, ret, opsize = -1;
 	static csh handle = 0;
 	static int omode = -1;
@@ -177,7 +177,7 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 
 	int mode = a->big_endian ? CS_MODE_BIG_ENDIAN : CS_MODE_LITTLE_ENDIAN;
 
-	//mode |= (a->bits==64)? CS_MODE_64: CS_MODE_32;
+	// mode |= (a->bits==64)? CS_MODE_64: CS_MODE_32;
 	if (mode != omode || a->bits != obits) {
 		cs_close(&handle);
 		handle = 0;
@@ -706,12 +706,12 @@ static int analop(RzAnalysis *a, RzAnalysisOp *op, ut64 addr, const ut8 *buf, in
 	}
 beach:
 	cs_free(insn, n);
-	//cs_close (&handle);
+	// cs_close (&handle);
 fin:
 	return opsize;
 }
 
-static bool set_reg_profile(RzAnalysis *analysis) {
+static char *get_reg_profile(RzAnalysis *analysis) {
 	const char *p =
 		"=PC    pc\n"
 		"=SP    a7\n"
@@ -736,37 +736,37 @@ static bool set_reg_profile(RzAnalysis *analysis) {
 		"gpr	a5	.32	52	0\n"
 		"gpr	a6 	.32	56	0\n"
 		"gpr	a7 	.32	60	0\n"
-		"gpr	fp0	.32	64	0\n" //FPU register 0, 96bits to write and read max
-		"gpr	fp1	.32	68	0\n" //FPU register 1, 96bits to write and read max
-		"gpr	fp2	.32	72	0\n" //FPU register 2, 96bits to write and read max
-		"gpr	fp3 	.32	76	0\n" //FPU register 3, 96bits to write and read max
-		"gpr	fp4 	.32	80	0\n" //FPU register 4, 96bits to write and read max
-		"gpr	fp5 	.32	84	0\n" //FPU register 5, 96bits to write and read max
-		"gpr	fp6 	.32	88	0\n" //FPU register 6, 96bits to write and read max
-		"gpr	fp7 	.32	92	0\n" //FPU register 7, 96bits to write and read max
+		"gpr	fp0	.32	64	0\n" // FPU register 0, 96bits to write and read max
+		"gpr	fp1	.32	68	0\n" // FPU register 1, 96bits to write and read max
+		"gpr	fp2	.32	72	0\n" // FPU register 2, 96bits to write and read max
+		"gpr	fp3 	.32	76	0\n" // FPU register 3, 96bits to write and read max
+		"gpr	fp4 	.32	80	0\n" // FPU register 4, 96bits to write and read max
+		"gpr	fp5 	.32	84	0\n" // FPU register 5, 96bits to write and read max
+		"gpr	fp6 	.32	88	0\n" // FPU register 6, 96bits to write and read max
+		"gpr	fp7 	.32	92	0\n" // FPU register 7, 96bits to write and read max
 		"gpr	pc 	.32	96	0\n"
-		"gpr	sr 	.32	100	0\n" //only available for read and write access during supervisor mode 16bit
-		"gpr	ccr 	.32	104	0\n" //subset of the SR, available from any mode
-		"gpr	sfc 	.32	108	0\n" //source function code register
-		"gpr	dfc	.32	112	0\n" //destination function code register
-		"gpr	usp	.32	116	0\n" //user stack point this is an shadow register of A7 user mode, SR bit 0xD is 0
-		"gpr	vbr	.32	120	0\n" //vector base register, this is a Address pointer
-		"gpr	cacr	.32	124	0\n" //cache control register, implementation specific
-		"gpr	caar	.32	128	0\n" //cache address register, 68020, 68EC020, 68030 and 68EC030 only.
-		"gpr	msp	.32	132	0\n" //master stack pointer, this is an shadow register of A7 supervisor mode, SR bits 0xD && 0xC are set
-		"gpr	isp	.32	136	0\n" //interrupt stack pointer, this is an shadow register of A7  supervisor mode, SR bit 0xD is set, 0xC is not.
+		"gpr	sr 	.32	100	0\n" // only available for read and write access during supervisor mode 16bit
+		"gpr	ccr 	.32	104	0\n" // subset of the SR, available from any mode
+		"gpr	sfc 	.32	108	0\n" // source function code register
+		"gpr	dfc	.32	112	0\n" // destination function code register
+		"gpr	usp	.32	116	0\n" // user stack point this is an shadow register of A7 user mode, SR bit 0xD is 0
+		"gpr	vbr	.32	120	0\n" // vector base register, this is a Address pointer
+		"gpr	cacr	.32	124	0\n" // cache control register, implementation specific
+		"gpr	caar	.32	128	0\n" // cache address register, 68020, 68EC020, 68030 and 68EC030 only.
+		"gpr	msp	.32	132	0\n" // master stack pointer, this is an shadow register of A7 supervisor mode, SR bits 0xD && 0xC are set
+		"gpr	isp	.32	136	0\n" // interrupt stack pointer, this is an shadow register of A7  supervisor mode, SR bit 0xD is set, 0xC is not.
 		"gpr	tc	.32	140	0\n"
-		"gpr	itt0	.32	144	0\n" //in 68EC040 this is IACR0
-		"gpr	itt1	.32	148	0\n" //in 68EC040 this is IACR1
-		"gpr	dtt0	.32	156	0\n" //in 68EC040 this is DACR0
-		"gpr	dtt1	.32	160	0\n" //in 68EC040 this is DACR1
+		"gpr	itt0	.32	144	0\n" // in 68EC040 this is IACR0
+		"gpr	itt1	.32	148	0\n" // in 68EC040 this is IACR1
+		"gpr	dtt0	.32	156	0\n" // in 68EC040 this is DACR0
+		"gpr	dtt1	.32	160	0\n" // in 68EC040 this is DACR1
 		"gpr	mmusr	.32	164	0\n"
 		"gpr	urp	.32	168	0\n"
 		"gpr	srp	.32	172	0\n"
 		"gpr	fpcr	.32	176	0\n"
 		"gpr	fpsr	.32	180	0\n"
 		"gpr	fpiar	.32	184	0\n";
-	return rz_reg_set_profile_string(analysis->reg, p);
+	return strdup(p);
 }
 
 RzAnalysisPlugin rz_analysis_plugin_m68k_cs = {
@@ -775,9 +775,9 @@ RzAnalysisPlugin rz_analysis_plugin_m68k_cs = {
 	.license = "BSD",
 	.esil = false,
 	.arch = "m68k",
-	.set_reg_profile = &set_reg_profile,
+	.get_reg_profile = &get_reg_profile,
 	.bits = 32,
-	.op = &analop,
+	.op = &analyze_op,
 };
 #else
 RzAnalysisPlugin rz_analysis_plugin_m68k_cs = {
