@@ -1679,8 +1679,7 @@ RZ_API const char *rz_line_readline_cb(RzLineReadCallback cb, void *user) {
 		case 6: // ^f // emacs right
 			__move_cursor_right();
 			break;
-		case 12: // ^L -- right
-			__move_cursor_right();
+		case 12: // ^L -- clear screen
 			if (I.echo) {
 				eprintf("\x1b[2J\x1b[0;0H");
 			}
@@ -1875,6 +1874,9 @@ RZ_API const char *rz_line_readline_cb(RzLineReadCallback cb, void *user) {
 					__print_prompt();
 					continue;
 				}
+				if (buf[0] == 'O' && strchr("ABCDFH", buf[1]) != NULL) { // O
+					buf[0] = '['; // 0x5b
+				}
 				if (buf[0] == 0x5b) { // [
 					switch (buf[1]) {
 					case '3': // supr
