@@ -229,8 +229,8 @@ RZ_API void rz_core_debug_breakpoint_toggle(RZ_NONNULL RzCore *core, ut64 addr)
 	if (bpi) {
 		rz_bp_del(core->dbg->bp, addr);
 	} else {
-		int hwbp = (int)rz_config_get_i(core->config, "dbg.hwbp");
-		bpi = rz_debug_bp_add(core->dbg, addr, hwbp, false, 0, NULL, 0);
+		bool hwbp = (int)rz_config_get_b(core->config, "dbg.hwbp");
+		bpi = rz_debug_bp_add(core->dbg, addr, hwbp ? 1 : 0, false, 0, NULL, 0);
 		if (!bpi) {
 			RZ_LOG_ERROR("core: cannot set breakpoint at 0x%" PFMT64x "\n", addr);
 		}
@@ -253,11 +253,11 @@ RZ_API void rz_core_debug_bp_add_noreturn_func(RzCore *core) {
 	RzBinSymbol *symbol;
 	RzListIter *iter;
 	RzBreakpointItem *bp;
-	int hwbp = rz_config_get_i(core->config, "dbg.hwbp");
+	bool hwbp = rz_config_get_b(core->config, "dbg.hwbp");
 	rz_list_foreach (symbols, iter, symbol) {
 		if (symbol->type && !strcmp(symbol->type, RZ_BIN_TYPE_FUNC_STR)) {
 			if (rz_analysis_noreturn_at(core->analysis, symbol->vaddr)) {
-				bp = rz_debug_bp_add(core->dbg, symbol->vaddr, hwbp, false, 0, NULL, 0);
+				bp = rz_debug_bp_add(core->dbg, symbol->vaddr, hwbp ? 1 : 0, false, 0, NULL, 0);
 				if (!bp) {
 					RZ_LOG_ERROR("Unable to add a breakpoint into a noreturn function %s at addr 0x%" PFMT64x "\n", symbol->name, symbol->vaddr);
 					return;
@@ -868,6 +868,7 @@ RZ_API void rz_backtrace_free(RZ_NULLABLE RzBacktrace *bt) {
 static void get_backtrace_info(RzCore *core, RzDebugFrame *frame, ut64 addr,
 	char **flagdesc, char **flagdesc2, char **pcstr, char **spstr) {
 	RzFlagItem *f = rz_flag_get_at(core->flags, frame->addr, true);
+	RzFlagItem *f2 = NULL;
 	*flagdesc = NULL;
 	*flagdesc2 = NULL;
 	if (f) {
@@ -884,20 +885,20 @@ static void get_backtrace_info(RzCore *core, RzDebugFrame *frame, ut64 addr,
 			*flagdesc = rz_str_newf("%s", f->name);
 		}
 		if (!strchr(f->name, '.')) {
-			f = rz_flag_get_at(core->flags, frame->addr - 1, true);
+			f2 = rz_flag_get_at(core->flags, frame->addr - 1, true);
 		}
-		if (f) {
-			if (f->offset != addr) {
-				int delta = (int)(frame->addr - 1 - f->offset);
+		if (f2 && f2 != f) {
+			if (f2->offset != addr) {
+				int delta = (int)(frame->addr - 1 - f2->offset);
 				if (delta > 0) {
-					*flagdesc2 = rz_str_newf("%s+%d", f->name, delta + 1);
+					*flagdesc2 = rz_str_newf("%s+%d", f2->name, delta + 1);
 				} else if (delta < 0) {
-					*flagdesc2 = rz_str_newf("%s%d", f->name, delta + 1);
+					*flagdesc2 = rz_str_newf("%s%d", f2->name, delta + 1);
 				} else {
-					*flagdesc2 = rz_str_newf("%s+1", f->name);
+					*flagdesc2 = rz_str_newf("%s+1", f2->name);
 				}
 			} else {
-				*flagdesc2 = rz_str_newf("%s", f->name);
+				*flagdesc2 = rz_str_newf("%s", f2->name);
 			}
 		}
 	}
